/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.vii.brillien.services.xmlrpc;

import com.vii.brillien.core.management.BrillienServices;
import com.vii.brillien.ignition.BrillienContext;
import com.vii.brillien.ignition.transport.BrillienCommunication;
import com.vii.brillien.ignition.transport.BrillienMediator;
import com.vii.brillien.kernel.axiom.sso.Session;
import com.vii.streamline.services.error.StreamLineException;
import com.vii.streamline.services.reflection.Invocation;
import com.vii.streamline.structures.collections.InnerList;
import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.XmlRpcHandler;
import org.apache.xmlrpc.XmlRpcRequest;
import org.slf4j.Logger;

import java.util.HashMap;

/**
 * Specialized XmlRpcHandler to able to delegate XML-Rpc calls to presences.
 */
public class BrillienXmlRpcHandler implements XmlRpcHandler {

    protected boolean                   ssoRequired;
    protected Logger                    logger;
    protected BrillienMediator          mediator;
    protected String                    handlerName;

    public BrillienXmlRpcHandler(boolean ssoRequired, Logger logger, BrillienMediator mediator, String handlerName) {
        this.ssoRequired = ssoRequired;
        this.logger = logger;
        this.mediator = mediator;
        this.handlerName = handlerName;
    }

    protected boolean isMethodReckognised(Invocation invocation) {
        return BrillienServices.isServiceExists(invocation);
    }

    @Override
    public Object execute(XmlRpcRequest xmlRpcRequest) throws XmlRpcException {
        Session session = ssoRequired ? Session.createSession( xmlRpcRequest.getParameter(0).toString() ) : null;

        Invocation invocation = new Invocation( session == null ? null : session.getId(), xmlRpcRequest.getMethodName() );

        try {
            if( !isMethodReckognised( invocation ) )
                throw new StreamLineException("Invocation's target cannot be identified.");

            logger.debug(
                "Calling Presence " +
                invocation.getEntity()
            );

            InnerList parameters = new InnerList(); //[ ssoRequired ? xmlRpcRequest.getParameterCount()-1 : xmlRpcRequest.getParameterCount() ];
            for( int i=ssoRequired?1:0; i<xmlRpcRequest.getParameterCount(); ++i ){
                parameters.add( xmlRpcRequest.getParameter( i ) );
            }

            BrillienCommunication c = mediator.sendGet(
                    session, BrillienContext.RECONNECTION_TIMEOUT,
                    invocation.getEntity(),
                    invocation.getMethod(),
                    parameters.asArray()
            );
            Object payload = c.acquireResponse();

            logger.debug(
                "Response received: " + payload
            );

            return payload;
        } catch (Exception e) {
            logger.error( e.getMessage(), e );
            throw new XmlRpcException( e.getMessage(), e );
        }
    }

}
